unit Bunny1;

{
  Progressive Mesh reduction demonstration (c)2001 by Paul TOTH <tothpaul@free.fr>

  based on

	Polygon Reduction Demo
	By Stan Melax (c) 1998
	mailto:melax@cs.ualberta.ca
	http://www.cs.ualberta.ca/~melax

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, WinGL, StdCtrls, ComCtrls, DelphiGL, ProgMesh, ExtCtrls;

type
  TNormal=record x,y,z:single end;
  TForm1 = class(TForm)
    WinGL1: TWinGL;
    TrackBar1: TTrackBar;
    procedure WinGL1Paint(Sender: TObject);
    procedure WinGL1CreateGL(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure WinGL1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure WinGL1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    Vertices :TVertices;
    Triangles:TTriangles;
    Simplify :TTriangles;
    Rotation :TVertices;
    lod:integer;
    procedure Rotate(rx,ry,rz:single);
    procedure Draw(Triangles:TTriangles);
  public
    { Public declarations }
    mx,my:integer;
    rx,ry:single;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
 RabbitData;

procedure TForm1.WinGL1CreateGL(Sender: TObject);
var
 i:integer;
 x1,y1,z1:single;
 x2,y2,z2:single;
 nx,ny,nz:single;
 vl:single;
begin
 SetLength(Vertices,rVertexCount);
 move(rVertices,Vertices[0],rVertexCount*sizeof(TVertex));
 SetLength(Triangles,rTriangleCount);
 move(rTriangles,Triangles[0],rTriangleCount*sizeof(TTriangle));
 Simplify:=Triangles;
 Caption:=IntTostr(Length(Vertices))+' vertices, '+IntToStr(Length(Triangles))+' faces';
 TrackBar1.Max:=Length(Vertices)-1;
 glEnable(GL_LIGHTING);
 glEnable(GL_LIGHT0);
 glColor3f(1,1,1);
end;

procedure TForm1.Rotate(rx,ry,rz:single);
var
 cx,cy,cz:single;
 sx,sy,sz:single;
 rx1,ry1,rz1:single;
 rx2,ry2,rz2:single;
 rx3,ry3,rz3:single;
 i:integer;
 begin
  rx:=rx*PI/180;
  ry:=ry*PI/180;
  rz:=rz*PI/180;
  cx:=cos(rx);
  cy:=cos(ry);
  cz:=cos(rz);
  sx:=sin(rx);
  sy:=sin(ry);
  sz:=sin(rz);
  rx1:= cy*cz; rx2:=sx*sy*cz-cx*sz; rx3:=cx*sy*cz+sx*sz;
  ry1:= cy*sz; ry2:=sx*sy*sz+cx*cz; ry3:=cx*sy*sz-sx*cz;
  rz1:=-sy;    rz2:=sx*cy;          rz3:=cx*cy;
  SetLength(Rotation,Length(Vertices));
  for i:=0 to Length(Vertices)-1 do with Vertices[i] do begin
   Rotation[i].x:=x*rx1+y*rx2+z*rx3;
   Rotation[i].y:=x*ry1+y*ry2+z*ry3;
   Rotation[i].z:=x*rz1+y*rz2+z*rz3;
  end;
 end;

procedure TForm1.WinGL1Paint(Sender: TObject);
var
 i:integer;
begin
 Rotate(rx,ry,0);
 glTranslatef(-1,0,-20);
 Draw(Triangles);

 if length(CollapsedFaces)=0 then exit;

 glDisable(GL_DEPTH_TEST);
 glDisable(GL_LIGHTING);
 glEnable(GL_BLEND);
 glBlendFunc(GL_ONE_MINUS_SRC_ALPHA,GL_SRC_ALPHA);
 glColor4f(0.8,0,0,0.75);
 glBegin(GL_TRIANGLES);
 for i:=0 to Length(CollapsedFaces)-1 do with Triangles[CollapsedFaces[i]] do begin
  with Rotation[a] do glVertex3f(x,y,z);
  with Rotation[b] do glVertex3f(x,y,z);
  with Rotation[c] do glVertex3f(x,y,z);
 end;
 glEnd;
 glDisable(GL_BLEND);
 glEnable(GL_DEPTH_TEST);
 glEnable(GL_LIGHTING);

// Rotate(0,-60,0);
 glTranslatef(2,0,0);
 Draw(Simplify);
end;

procedure TForm1.Draw(Triangles:TTriangles);
var
 i:integer;
 x1,y1,z1:single;
 x2,y2,z2:single;
 nx,ny,nz:single;
 vl:single;
begin
 glBegin(GL_TRIANGLES);
 for i:=0 to Length(Triangles)-1 do with Triangles[i] do begin
  x1:=Rotation[b].x-Rotation[a].x;
  y1:=Rotation[b].y-Rotation[a].y;
  z1:=Rotation[b].z-Rotation[a].z;
  x2:=Rotation[c].x-Rotation[a].x;
  y2:=Rotation[c].y-Rotation[a].y;
  z2:=Rotation[c].z-Rotation[a].z;
  nx:=y1*z2-y2*z1;
  ny:=z1*x2-z2*x1;
  nz:=x1*y2-x2*y1;
  vl:=sqrt(nx*nx+ny*ny+nz*nz);
  if vl<>0 then begin
   nx:=nx/vl;
   ny:=ny/vl;
   nz:=nz/vl;
  end;
  glNormal3f(nx,ny,nz);
  with Rotation[a] do glVertex3f(x,y,z);
  with Rotation[b] do glVertex3f(x,y,z);
  with Rotation[c] do glVertex3f(x,y,z);
 end;
 glEnd;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
 if lod<>TrackBar1.Position then begin
  lod:=TrackBar1.Position;
  Simplify:=ProgressiveMesh(Vertices,Triangles,lod);
  Caption:=IntTostr(Length(Vertices))+' vertices, '+IntToStr(Length(Triangles))+' faces'+
     '-->'+IntToStr(TrackBar1.Position)+', '+IntToStr(Length(Simplify))+' faces';
  WinGL1.Invalidate;
 end; 
end;

procedure TForm1.WinGL1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 mx:=x;
 my:=y;
end;

procedure TForm1.WinGL1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 if ssLeft in Shift then begin
  rx:=rx+(y-my);
  ry:=ry+(x-mx);
  wingl1.invalidate;
 end;
 mx:=x;
 my:=y;
end;

end.

