unit ProgMesh;
{
  Progressive Mesh reduction demonstration (c)2001 by Paul TOTH <tothpaul@free.fr>

  based on

	Polygon Reduction Demo
	By Stan Melax (c) 1998
	mailto:melax@cs.ualberta.ca
	http://www.cs.ualberta.ca/~melax

}
{-$define log}
interface

uses
{$ifdef log}windows,{$endif}
 classes,
 sysutils;

Type
 TVertex=record
  x,y,z:single;
 end;
 TVertices=array of TVertex;

 TTriangle=record
  a,b,c:integer;
 end;
 TTriangles=array of TTriangle;

// return an array of TTriangle that use "lod" vertices. with (lod>=3) and (lod<length(AVertices))
function ProgressiveMesh(AVertices:TVertices; ATriangles:TTriangles; lod:integer):TTriangles;

var
 Collapsedfaces:array of integer;

implementation

Const
 BIG=3.4e34-1;

Type
 TIntegers=array of Integer;

 TNeighbor=record
  index:integer;
  count:integer;
 end;
 TNeighbors=array of TNeighbor;

 TVertexInfo=record
  Neighbors:TNeighbors; { Vertex }
  Faces    :TIntegers; { Triangle }
  Collapse :integer;
  Distance :single;
 end;
 TVertexInfos=array of TVertexInfo;

 TFaceInfo=record
  x,y,z:single;
  collapsed:boolean;
 end;
 TFaceInfos=array of TFaceInfo;

var
 Vertices:TVertices;
 Triangles:TTriangles;
 VertexInfos:TVertexInfos;
 FaceInfos:TFaceInfos;

// add a neighbor to a vertex
procedure AddNeighbor(vertex, Neighbor:integer);
 var
  i:integer;
 begin
  if vertex=Neighbor then exit;
  with VertexInfos[vertex] do begin
   for i:=0 to Length(Neighbors)-1 do
    if Neighbors[i].Index=Neighbor then begin
     inc(Neighbors[i].Count);
     exit;
    end;
   i:=Length(Neighbors);
   SetLength(Neighbors,i+1);
   Neighbors[i].Index:=Neighbor;
  end;
 end;

procedure DelNeighbor(vertex, Neighbor:integer);
 var
  l,i:integer;
 begin
  if vertex=Neighbor then exit;
  with VertexInfos[vertex] do begin
   l:=Length(Neighbors)-1;
   for i:=0 to l do begin
    if Neighbors[i].Index=Neighbor then begin
     if Neighbors[i].Count>0 then
      dec(Neighbors[i].Count)
     else begin
      Neighbors[i]:=Neighbors[l];
      SetLength(Neighbors,l);
     {$ifdef log}writeln(' -suppression du vertex ',neighbor,' au vertex ',vertex);{$endif}
     end;
     exit;
    end;
   end;
   {$ifdef log}beep;writeln(' -le vertex ',neighbor,' n''est pas dans le vertex ',vertex);{$endif}
  end;
 end;

// replace a neighbor
procedure NewNeighbor(vertex, oldneighbor,newneighbor:integer);
 var
  l,i,j:integer;
 begin
  if (vertex=oldneighbor)or(vertex=newneighbor)or(oldneighbor=newneighbor) then exit;
  DelNeighbor(vertex,oldNeighbor);
  AddNeighbor(vertex,NewNeighbor);
 end;

// add a triangle to a vertex
procedure AddTriangle(vertex,triangle:integer);
 var
  i:integer;
 begin
  with VertexInfos[vertex] do begin
   i:=Length(Faces);
   SetLength(Faces,i+1);
   Faces[i]:=triangle;
   with Triangles[triangle] do begin
    AddNeighbor(vertex,a);
    AddNeighbor(vertex,b);
    AddNeighbor(vertex,c);
   end;
  end;
 end;

// remove a triangle from a vertex
procedure DelTriangle(vertex,triangle:integer);
 var
  i,l:integer;
 begin
  with VertexInfos[vertex] do begin
   if Distance>BIG then exit;
  {$ifdef log}
   write(' vertex ',vertex,' (');
   for i:=0 to length(faces)-1 do write(' ',faces[i]);
   write(') -> (');
  {$endif}
   l:=Length(Faces)-1;
   i:=l;
   while i>=0 do begin
    if Faces[i]=triangle then begin
     Faces[i]:=Faces[l];
     SetLength(Faces,l);
  {$ifdef log}
   for i:=0 to length(faces)-1 do write(' ',faces[i]);
   writeln(')');
  {$endif}
     // suppression du vertex si plus utilis
     with Triangles[triangle] do begin
      DelNeighbor(vertex,a);
      DelNeighbor(vertex,b);
      DelNeighbor(vertex,c);
     end;
     exit;
    end;
    dec(i);
   end;
  {$ifdef log}
   beep;
   writeln('face ',triangle,' non trouve sur le point ',vertex);
  {$endif}
  end;
 end;

// normals dot product
function dotproduct(a,b:integer):single;
 begin
  result:=FaceInfos[a].x*FaceInfos[b].x
         +FaceInfos[a].y*FaceInfos[b].y
         +FaceInfos[a].z*FaceInfos[b].z;
 end;

function CollapseCost(u,v:integer):single;
 var
  dx,dy,dz:single;
  len:single;
  sides:array of integer;
  scount:integer;
  i,j,f:integer;
  curvature:single;
  mincurv:single;
  curv:single;
 begin
  dx:=Vertices[u].x-Vertices[v].x;
  dy:=Vertices[u].y-Vertices[v].y;
  dz:=Vertices[u].z-Vertices[v].z;
  len:=sqrt(dx*dx+dy*dy+dz*dz);
  curvature:=0;
  scount:=0;
  with VertexInfos[u] do begin

   SetLength(sides,Length(Faces));

   for i:=0 to Length(Faces)-1 do begin
    f:=Faces[i];
    with Triangles[f] do begin
     if (a=v)or(b=v)or(c=v) then begin
      sides[scount]:=f;
      inc(scount);
     end;
    end;
   end;

   for i:=0 to Length(Faces)-1 do begin
    mincurv:=1; // curve for face i and closer side to it
    for j:=0 to scount-1 do begin
     curv:=(1-dotproduct(faces[i],sides[j]))/2;
     if curv<mincurv then mincurv:=curv;
    end;
    if mincurv>curvature then curvature:=mincurv;
   end;

   sides:=nil;
  end;

  result:=len*curvature;
 end;

procedure ComputeNormal(Index:integer);
var
  x1,y1,z1:single;
  x2,y2,z2:single;
  nx,ny,nz:single;
  vl:single;
begin
 with Triangles[Index] do begin
  x1:=Vertices[b].x-Vertices[a].x;
  y1:=Vertices[b].y-Vertices[a].y;
  z1:=Vertices[b].z-Vertices[a].z;
  x2:=Vertices[c].x-Vertices[a].x;
  y2:=Vertices[c].y-Vertices[a].y;
  z2:=Vertices[c].z-Vertices[a].z;
  nx:=y1*z2-y2*z1;
  ny:=z1*x2-z2*x1;
  nz:=x1*y2-x2*y1;
  vl:=sqrt(nx*nx+ny*ny+nz*nz);
  if vl<>0 then begin
   nx:=nx/vl;
   ny:=ny/vl;
   nz:=nz/vl;
  end;
 end;
 with FaceInfos[Index] do begin
  x:=nx;
  y:=ny;
  z:=nz;
 end;
end;

procedure ComputeVertexCost(Index:integer);
var
 i,j:integer;
 l:single;
begin
 with VertexInfos[Index] do begin
  if Distance>BIG then exit; // this point is no more used
  if Length(Neighbors)=0 then begin
  {$ifdef log}beep; writeln('vertex ',index,' have no Neighbor ?!');{$endif}
   Collapse:=-1;
   Distance:=0;
  end else begin
   Collapse:=Neighbors[0].index;
   Distance:=CollapseCost(Index,Collapse);
   for i:=1 to Length(Neighbors)-1 do begin
    j:=Neighbors[i].index;
    l:=CollapseCost(Index,j);
    if l<Distance then begin
     Collapse:=j;
     Distance:=l;
    end;
   end;
  end;
 end;
end;

function CollapseVertex:integer;
var
 i,j,f,n:integer;
 d,d2:single;
begin
 {$ifdef log}writeln;{$endif}
  Result:=0;
 // find a vertex to collapse
  i:=0;
  d:=VertexInfos[i].Distance;
  for j:=1 to Length(Vertices)-1 do begin
   d2:=VertexInfos[j].Distance;
   if d2<d then begin
    d:=d2;
    i:=j;
   end;
  end;
  if d>BIG then raise Exception.Create('impossible de rduire la figure');
 // collapse selected vertex
  with VertexInfos[i] do begin
  {$ifdef log}
   write('remplacer le vertex ',i,' (');
   for j:=0 to length(Faces)-1 do write(' ',faces[j]);
   writeln(')');
   write(' par le vertex ',collapse,' (');
   for j:=0 to length(VertexInfos[collapse].Faces)-1 do write(' ',VertexInfos[collapse].faces[j]);
   writeln(')');
  {$endif}
   Distance:=BIG+1;
   for j:=0 to Length(Faces)-1 do begin
    f:=Faces[j];
    with Triangles[f] do begin
     if (a=Collapse)or(b=Collapse)or(c=Collapse) then begin
      {$ifdef log}writeln('suppression de la face ',f,' (',a,',',b,',',c,')');{$endif}
      if FaceInfos[f].Collapsed then raise exception.Create('face dj supprime !');
      FaceInfos[f].Collapsed:=true;
      DelTriangle(a,f);
      DelTriangle(b,f);
      DelTriangle(c,f);
      inc(Result);
     end else begin
      {$ifdef log}write('modification de la face ',f,' (',a,',',b,',',c,') -> (');{$endif}
      if a=i then a:=Collapse else
      if b=i then b:=Collapse else
      if c=i then c:=Collapse else raise exception.create('internal error');
      {$ifdef log}writeln(a,',',b,',',c,')');{$endif}
      AddTriangle(collapse,f);
      NewNeighbor(a,i,collapse);
      NewNeighbor(b,i,collapse);
      NewNeighbor(c,i,collapse);
      ComputeNormal(f);
     end;
    end;
   end;
   for j:=0 to Length(Neighbors)-1 do ComputeVertexCost(Neighbors[j].Index);
  end;
end;

// not usefull in the bunny demo...needed for multi-object meshes like Poser wavefront...
procedure FreezeBorders;
 var
  i,j,k:integer;
  point:integer;
  count:integer;
 begin
  for i:=0 to Length(VertexInfos)-1 do begin
   with VertexInfos[i] do begin
    for j:=0 to Length(Neighbors)-1 do begin
     point:=Neighbors[j].index;
     count:=0;
     for k:=0 to Length(Faces)-1 do begin
      with Triangles[Faces[k]] do if (a=point)or(b=point)or(c=point) then inc(count);
     end;
     if count<2 then Distance:=BIG+1;
    end;
   end;
  end;
 end;

function ProgressiveMesh(AVertices:TVertices; ATriangles:TTriangles; lod:integer):TTriangles;
 var
  i,n,j:integer;
 begin
 // global instance of Vertices
  Vertices:=AVertices;
 // copy of traingles, cause we will change them
  SetLength(Triangles,Length(ATriangles));
  Move(ATriangles[0],Triangles[0],Length(Triangles)*sizeof(TTriangle));
 // some more info
  SetLength(VertexInfos,Length(Vertices));
  SetLength(FaceInfos,Length(Triangles));
 // compute additionnal infos
  for i:=0 to Length(Triangles)-1 do begin
   with Triangles[i] do begin
    AddTriangle(a,i);
    AddTriangle(b,i);
    AddTriangle(c,i);
   end;
   ComputeNormal(i);
  end;
 // do not allow Border removing
  FreezeBorders;
 // compute collapse cost for each vertice
  for i:=0 to Length(Vertices)-1 do ComputeVertexCost(i);
 // collapse
  {$ifdef log}writeln('lod=',lod);{$endif}
  j:=0;
  for i:=0 to lod-1 do inc(j,CollapseVertex);
  SetLength(Result,Length(Triangles)-j);
  SetLength(collapsedFaces,j);
  n:=0;
  j:=0;
  for i:=0 to Length(Triangles)-1 do begin
   if FaceInfos[i].Collapsed then begin
    collapsedFaces[j]:=i;
    inc(j);
   end else begin
    Result[n]:=Triangles[i];
    inc(n);
   end;
  end;
  VertexInfos:=nil;
  FaceInfos:=nil;
  Triangles:=nil;
 end;

{$ifdef log}
initialization
 allocconsole;
{
 assignfile(output,'bunny.log');
 rewrite(output);
finalization
 closefile(output);
}
{$endif}
end.
